# -*- coding: utf-8 -*-
#
# File: mediafile.py
#
# Copyright (c) 2006 by Christof Haemmerle :: nex9 (USA), Andy Nicholson,
# Wolfgang Reutz
# Generator: ArchGenXML Version 1.5.0 svn/devel
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Christof Haemmerle <reco@nex9.com>, Andy Nicholson <andy@cat.org.au>,
Wolfgang Reutz <wolfgang.reutz@vol.at>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
import zope
from Products.ATContentTypes.content.file import ATFile
from Products.ATMediaFile.interfaces.IATMediaFile import IATMediaFile
from Products.ATMediaFile.interfaces.IMediaMetaData import IMediaMetaData
from Products.ATMediaFile.interfaces.ISyndication import IATMediaFileFeedEntry
from Products.ATMediaFile.interfaces.ISyndication import IATMediaFileFeedSource
from Products.ATMediaFile.config import *

# additional imports from tagged value 'import'
from Products.ATContentTypes.content.file import ATFileSchema
from Products.ExternalStorage.ExternalStorage import ExternalStorage
from Products.validation.validators.SupplValidators import MaxSizeValidator
from Products.validation import V_REQUIRED

##code-section module-header #fill in your manual code here
from Products.ATMediaFile.content.enclosure import ATMediaFileEnclosure
from Products.CMFCore.utils import getToolByName
import zLOG 
##/code-section module-header

schema = Schema((

    FileField(
        name='file',
        widget=FileWidget
        (
            label="Media File",
            description="Upload your media file.",
            label_msgid='ATMediaFile_label_file',
            description_msgid='ATMediaFile_help_file',
            i18n_domain='ATMediaFile',
        ),
        languageIndependent=1,
        storage=ExternalStorage(prefix=FS_DIRECTORY, archive=False, rename=True),
        primary=1,
        required=1
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

ATMediaFile_schema = getattr(ATFile, 'schema', Schema(())).copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class ATMediaFile(ATFile):
    """
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(ATFile,'__implements__',()),) + (IATMediaFile,) + (IMediaMetaData,)
    # zope3 interfaces
    zope.interface.implements(IATMediaFileFeedEntry, IATMediaFileFeedSource)

    allowed_content_types = [] + list(getattr(ATFile, 'allowed_content_types', []))
    _at_rename_after_creation = True

    schema = ATMediaFile_schema

    ##code-section class-header #fill in your manual code here

    filesize_cache = None
    ##/code-section class-header

    # Methods
    security.declarePublic('getDownloadServerURL')
    def getDownloadServerURL(self):
    	"""
		rewrite the absolute_url of this item to go via a remote host
		does not put any 'views' on the end of the URL! 
		it simply assumes linking straight to the file name will work
	"""
	if self.isUsingRemoteServer():	
		download_host = getToolByName(self,'portal_atmediafiletool').getRemote_download_server_url() 
		# assuming the string to contain ..
		# http://some.machine.net:1234/
		# etc 
		# protocol://mach.ine:xyz/fold/er/path/
		#
		return download_host + '/'.join(getToolByName(self, 'portal_url').getRelativeContentPath(self)[:-1])
	else:
		#if not defined, just return the url for the zope instance
		return self.absolute_url()
 
    security.declarePublic('getDownloadServerURL')
    def isUsingRemoteServer(self):
    	if getToolByName(self,'portal_atmediafiletool').getEnable_remote_downloadserver():
		return True
	else:
		return False

    # Methods from Interface IMediaMetaData

    security.declarePublic('getMetaTitle')
    def getMetaTitle(self):
        """
        returns the basic metatdata title
        """
        
        pass

    security.declarePublic('setMetaTitle')
    def setMetaTitle(self):
        """
        sets the basic metadata title
        """
        
        pass

    security.declarePublic('getMetaAuthor')
    def getMetaAuthor(self):
        """
        sets the author in the metadata
        """
        
        pass

    security.declarePublic('setMetaAuthor')
    def setMetaAuthor(self):
        """
        sets the author in the metadata
        """
        
        pass

    security.declarePublic('getMetaLicense')
    def getMetaLicense(self):
        """
        
        """
        
        pass

    security.declarePublic('setMetaLicense')
    def setMetaLicense(self):
        """
        
        """
        
        pass

    security.declarePublic('getFileAttribs')
    def getFileAttribs(self):
        """
       	Return an tuple describing the media file
        """
	#get the ExternalStorage object for the file
        file_ext_storage_obj = self.Schema()['file'].storage
        #get the URL
	# below, returns a relative URL
        #url = file_ext_storage_obj.getInstancePath(self)
	# below, returns a absolute URL
	url = self.absolute_url() + "/download"
        #type
        type = self.Schema()['file'].getContentType(self)
        #length of enclosure
	if self.filesize_cache is None or self.filesize_cache == 0:
        	length = self.Schema()['file'].get_size(self)
		zLOG.LOG('getFileAttribs',zLOG.INFO, 'using the file system , length %s' % (length))
		self.filesize_cache = length
	else: 
		length = self.filesize_cache
		zLOG.LOG('getFileAttribs',zLOG.INFO, 'using the CACHE of file system , length %s' % (length))

	return (url, length, type)

    # Methods from Interface IATMediaFileFeedEntry

    security.declarePublic('getEnclosure')
    def getEnclosure(self):
        """
       	Return an IEnclosure adapting object 
        """
	(url, length, type) = self.getFileAttribs()
	zLOG.LOG('getEnclosure',zLOG.DEBUG, 'url %s length %s type %s' % (url, length,type))
        return ATMediaFileEnclosure(url, length, type)

    # Methods from Interface IATMediaFileFeedSource

    security.declarePublic('getFeedEntries')
    def getFeedEntries(self):
        """
        Return myself as a list of one, representing a list of IFeedEntries
        """
        return [self]

    # Manually created methods

    security.declarePublic('getMetaContactEmail')
    def getMetaContactEmail(self):
        """
        
        """
        
        pass

    security.declarePublic('setMetaHomePage')
    def setMetaHomePage(self):
        """
        
        """
        
        pass

    security.declarePublic('setMetaProducedDate')
    def setMetaProducedDate(self):
        """
        
        """
        
        pass

    security.declarePublic('setMetaContactEmail')
    def setMetaContactEmail(self):
        """
        
        """
        
        pass

    security.declarePublic('getMetaHomePage')
    def getMetaHomePage(self):
        """
        
        """
        
        pass

    security.declarePublic('getMetaProducedDate')
    def getMetaProducedDate(self):
        """
        
        """
        
        pass


# end of class ATMediaFile

##code-section module-footer #fill in your manual code here
##/code-section module-footer



