
import os
import tempfile
from Products.PloneTestCase import PloneTestCase
from Products.ExternalStorage.config import ENVIRONMENT_VARIABLE
from Products.CMFPlone import transaction

tempdir = tempfile.gettempdir()

FILENAME_TEXT = 'text.file'
FILENAME_TEXT2 = 'moretext.file'
FILENAME_TEXT_NEW = 'new.file'

CONTENT_TXT = """mytestfile"""

DATAPATH = 'data/'
DATAPATH_TEXT = os.path.join( DATAPATH, FILENAME_TEXT )
DATAPATH_TEXT2 = os.path.join( DATAPATH, FILENAME_TEXT2 )

STORAGE_PATH = os.path.join( tempdir, 'atmediafile_tests' )
BASE_PATH = os.path.join( STORAGE_PATH, 'files', 'Members' )


class ATMFTestCase(PloneTestCase.PloneTestCase):
    """ Base ATMediaFile test case.  Defines some helper methods.
    """

    def beforeTearDown(self):
        """ Remove all the added files.
        """
        base = os.environ.get(ENVIRONMENT_VARIABLE)
        for root, dirs, files in os.walk(base, topdown=None):
            for name in files:
                os.remove(os.path.join(root, name))
            for name in dirs:
                os.rmdir(os.path.join(root, name))
        if os.path.exists(base):
            os.rmdir(base)

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
	# This modifies the path ExternalStorage will use to store files
	#   (instead of $INSTANCE_PATH/var/files/ ).
        os.environ[ENVIRONMENT_VARIABLE] = STORAGE_PATH

    def _addFileByString(self, place, field, id):
        """Adds a file by string.
        """
        place.invokeFactory('ATMediaExample', id=id)
        obj = getattr(place, id)
        transaction.commit(1)
        kw = {field: CONTENT_TXT}
        obj.edit(**kw)
        return obj

    def _addFileByFileUpload(self, place, field, id,
                             portal_type='ATMediaExample'):
        """Adds a file by file upload.
        """
        place.invokeFactory(portal_type, id=id)
        obj = getattr(place, id)
        transaction.commit(1)
        self._updateContent(obj, field, DATAPATH + id)
        return obj

    def _updateContent(self, obj, field, filepath):
        """Updates a field content for a file.
        """
        from Products.ExternalStorage.tests.dummy import FileUpload
        file = open(filepath, 'r')
        file.seek(0)
        filename = filepath.split('/')[-1]
        fu = FileUpload(filename=filename, file=file)
        kw = {field: fu}
        obj.edit(**kw)

