# File: testATMediaFile_add.py
# 
# Copyright (c) 2005 by Chris Miles
# Generator: manual - this couldn't be generated by AGX as it needs to
#     use ATMediaExample to perform the ATMediaFile tests.
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#
__author__  = '''Chris Miles <miles.chris@gmail.com>'''
__docformat__ = 'plaintext'

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import ATMFTestCase
from OFS.Image import File

#
# Base TestCase for ATMediaFile
#
from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase

ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PortalTransforms', quiet=1)
ZopeTestCase.installProduct('MimetypesRegistry', quiet=1)
ZopeTestCase.installProduct('ATMediaFile')
ZopeTestCase.installProduct('ATMediaExample')	## Added manually...
# If the products's config includes DEPENDENCIES, install them too
try:
    from Products.ATMediaFile.config import DEPENDENCIES
except:
    DEPENDENCIES = []
for dependency in DEPENDENCIES:
    ZopeTestCase.installProduct(dependency)

#PRODUCTS = ('Archetypes', 'ATMediaFile')
PRODUCTS = ('Archetypes', 'ATMediaExample')

PloneTestCase.setupPloneSite(products=PRODUCTS)

class TestAddFileViaString(ATMFTestCase.ATMFTestCase):
    """ TestCase for ATMediaFile: add media file using string of text """

    def afterSetUp(self):
	# Add objects to the workarea
	self.textfile = self._addFileByString(self.folder,
		'file',
		ATMFTestCase.FILENAME_TEXT)
	self.basepath = os.path.join( ATMFTestCase.BASE_PATH,
		PloneTestCase.default_user )
	self.path = os.path.join( self.basepath,
		ATMFTestCase.FILENAME_TEXT )

    def testAddedObjectId(self):
        self.assertEqual(ATMFTestCase.FILENAME_TEXT, self.textfile.getId())

    def testAddedObjectContent(self):
        file_obj = self.textfile.getFile()
        self.assertEqual(ATMFTestCase.CONTENT_TXT, file_obj.data)

    def testAddedTypeContentOnZODB(self):
        data = self.textfile.getFile()
        self.failUnless(isinstance(data, File))

    def testAddedFileContent(self):
        self.assertEqual(ATMFTestCase.CONTENT_TXT, open(self.path, 'r').read())

    def testAddedObjectPath(self):
        field = self.textfile.getField('file')
        self.assertEqual(self.path, field.storage.getFilepath(self.textfile, 'file'))

    def testAddedFileSystemPresence(self):
        self.failUnless(ATMFTestCase.FILENAME_TEXT in
                        os.listdir( self.basepath ))



class TestAddFileViaUploadFile(ATMFTestCase.ATMFTestCase):
    """ TestCase for ATMediaFile: add media file using FileUpload. """

    def afterSetUp(self):
        self.art = self._addFileByFileUpload(self.folder, 'file',
                                             ATMFTestCase.FILENAME_TEXT)
	self.basepath = os.path.join( ATMFTestCase.BASE_PATH,
		PloneTestCase.default_user )
	self.path = os.path.join( self.basepath,
		ATMFTestCase.FILENAME_TEXT )

    def testAddedObjectId(self):
        self.assertEqual(ATMFTestCase.FILENAME_TEXT, self.art.getId())

    def testAddedObjectContent(self):
	fp = open(ATMFTestCase.DATAPATH_TEXT, 'r')
	content = fp.read()
	fp.close()
        file_obj = self.art.getFile()
        self.assertEqual(content, file_obj.data)

    def testAddedTypeContentOnZODB(self):
        file_obj = self.art.getFile()
        self.failUnless(isinstance(file_obj, File))

    def testAddedFileContent(self):
	fp = open(ATMFTestCase.DATAPATH_TEXT, 'r')
	content1 = fp.read()
	fp.close()
	fp = open(self.path, 'r')
	content2 = fp.read()
	fp.close()
        self.assertEqual( content1, content2 )

    def testAddedObjectPath(self):
        field = self.art.getField('file')
        self.assertEqual(self.path, field.storage.getFilepath(self.art, 'file'))

    def testAddedFileSystemPresence(self):
        self.failUnless(ATMFTestCase.FILENAME_TEXT in
                        os.listdir( self.basepath ))



class TestMultipleAddFile(ATMFTestCase.ATMFTestCase):
    """ TestCase for ATMediaFile: add multiple media files. """

    def afterSetUp(self):
        self.art1 = self._addFileByFileUpload(self.folder, 'file',
                                             ATMFTestCase.FILENAME_TEXT)
        self.folder.invokeFactory('ATMediaExample', id='art2')
        self.art2 = self.folder.art2
	self.basepath = os.path.join( ATMFTestCase.BASE_PATH,
		PloneTestCase.default_user )

    def testAddedObjectId(self):
        self.assertEqual(ATMFTestCase.FILENAME_TEXT, self.art1.getId())
        self.assertEqual('art2', self.art2.getId())

    def testAddedObjectContent(self):
	fp = open(ATMFTestCase.DATAPATH_TEXT, 'r')
	content = fp.read()
	fp.close()
        file1_obj = self.art1.getFile()
        file2_obj = self.art2.getFile()
        self.assertEqual(content, str(file1_obj))
        self.assertEqual('', str(file2_obj))

    def testAddedTypeContentOnZODB(self):
        file1_obj = self.art1.getFile()
        file2_obj = self.art2.getFile()
        self.failUnless(isinstance(file1_obj, File))
        self.failUnless(isinstance(file2_obj, File))

    def testAddedObjectPath(self):
        field1 = self.art1.getField('file')
        field2 = self.art2.getField('file')
        path1 = os.path.join(self.basepath, ATMFTestCase.FILENAME_TEXT)
        path2 = os.path.join(self.basepath, 'art2')
        self.assertEqual(path1, field1.storage.getFilepath(self.art1, 'file'))
        self.assertEqual(path2, field2.storage.getFilepath(self.art2, 'file'))

    def testAddedFileSystemPresence(self):
        self.failUnless(ATMFTestCase.FILENAME_TEXT in
                        os.listdir(self.basepath))
        self.failUnless('art2' in
                        os.listdir(self.basepath))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestAddFileViaString))
    suite.addTest(makeSuite(TestAddFileViaUploadFile))
    suite.addTest(makeSuite(TestMultipleAddFile))
    return suite



if __name__ == '__main__':
    framework()


