# -*- coding: utf-8 -*-
#
# File: ATMediaFileTool.py
#
# Copyright (c) 2006 by Wolfgang Reutz
# Generator: ArchGenXML Version 1.5.0 svn/devel
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Wolfgang Reutz <wolfgang.reutz@vol.at>, Andy Nicholson <andy@engagemedia.org>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.ATMediaFile.interfaces.IATMediaFileTool import IATMediaFileTool
from Products.ATMediaFile.config import *


from Products.CMFCore.utils import UniqueObject

    
##code-section module-header #fill in your manual code here
import zLOG, transaction, traceback

#for BitTorrent support
#
try:
    from BitTorrent.platform import install_translation
    install_translation()
    from BitTorrent.makemetafile import make_meta_files
except:
    pass

#FIXME 
#BitTorrent.version 
# make sure major.minor >= 4.4

#file manipulation
import os
##/code-section module-header

schema = Schema((

    BooleanField(
        name='enable_bittorrent',
        default=0,
        widget=BooleanWidget(
            label="Enable BitTorrent support",
            description="Enables global BitTorrent support for all ATMediaFile based products (ATAudio, ATVideo, ...)",
            label_msgid='ATMediaFile_label_enable_bittorrent',
            description_msgid='ATMediaFile_help_enable_bittorrent',
            i18n_domain='ATMediaFile',
        )
    ),
    StringField(
        name='bittorrent_tracker_url',
        widget=StringWidget(
            label="BitTorrent tracker URL",
            description="Enter the URL of your BitTorrent tracker here. Will be used to create any new .torrent file.",
            size="50",
            label_msgid='ATMediaFile_label_bittorrent_tracker_url',
            description_msgid='ATMediaFile_help_bittorrent_tracker_url',
            i18n_domain='ATMediaFile',
        )
    ),
    BooleanField(
        name='enable_remote_bittorrent',
        default=0,
        widget=BooleanWidget(
            label="Enable Remote BitTorrent support",
            description="Enables global remote server BitTorrent support for all ATMediaFile based products (ATAudio, ATVideo, ...) Downloads media file, creates metadata file (.torrent) and starts seeding process.",
            label_msgid='ATMediaFile_label_enable_bittorrent',
            description_msgid='ATMediaFile_help_enable_bittorrent',
            i18n_domain='ATMediaFile',
        )
    ),

    StringField(
        name='remote_bittorrent_server_url',
        widget=StringWidget(
            label="Remote server hostname and portnumber",
            description="Enter the hostname and port number to contact the remote server. Enter in format hostname:portnumber",
            size="50",
            label_msgid='ATMediaFile_label_bittorrent_tracker_url',
            description_msgid='ATMediaFile_help_bittorrent_tracker_url',
            i18n_domain='ATMediaFile',
        )
    ),
     BooleanField(
        name='enable_remote_downloadserver',
        default=0,
        widget=BooleanWidget(
            label="Enable Remote Download Server support",
            description="Enables remote download server support for all ATMediaFile based products (ATAudio, ATVideo, ...). This will rewrite all URLs for downloading to go via the host given below.",
            label_msgid='ATMediaFile_label_enable_downloadserver',
            description_msgid='ATMediaFile_help_enable_downloadserver',
            i18n_domain='ATMediaFile',
        )
    ),

    StringField(
        name='remote_download_server_url',
        widget=StringWidget(
            label="Remote server hostname and portnumber",
            description="Enter the hostname and port number to contact the remote server. Enter in format hostname:portnumber",
            size="50",
            label_msgid='ATMediaFile_label_download_url',
            description_msgid='ATMediaFile_help_download_url',
            i18n_domain='ATMediaFile',
        )
    ),
   
),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

ATMediaFileTool_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class ATMediaFileTool(UniqueObject, BaseContent):
    """
    """
    security = ClassSecurityInfo()
    __implements__ = (getattr(UniqueObject,'__implements__',()),) + (getattr(BaseContent,'__implements__',()),) + (IATMediaFileTool,)

    # This name appears in the 'add' box
    archetype_name = 'ATMediaFileTool'

    meta_type = 'ATMediaFileTool'
    portal_type = 'ATMediaFileTool'
    allowed_content_types = []
    filter_content_types = 0
    global_allow = 0
    allow_discussion = False
    #content_icon = 'ATMediaFileTool.gif'
    immediate_view = 'base_view'
    default_view = 'base_view'
    suppl_views = ()
    typeDescription = "ATMediaFileTool"
    typeDescMsgId = 'description_edit_atmediafiletool'
    #toolicon = 'ATMediaFileTool.gif'

    _at_rename_after_creation = True

    schema = ATMediaFileTool_schema

    ##code-section class-header #fill in your manual code here
    ##/code-section class-header


    # tool-constructors have no id argument, the id is fixed
    def __init__(self, id=None):
        BaseContent.__init__(self,'portal_atmediafiletool')
        self.setTitle('ATMediaFileTool')
        
        ##code-section constructor-footer #fill in your manual code here
        ##/code-section constructor-footer


    # Methods
    security.declarePublic('createRemoteTorrentFile')
    def createRemoteTorrentFile(self,file_obj):
        """
        make a call via twistd perspective broker client to remote server, which will trigger a file download, torrent metadata file creation (.torrent)
	and seeding
        """
	#check if remote bittorrent support has been enabled.
	if (self.getEnable_remote_bittorrent() and self.getRemoteTorrentSupport()):
		zLOG.LOG('ATMediaFile', zLOG.INFO, 'making pb client call to remote server.. ')

		# get the ExternalStorage object for the video file
            	file_ext_storage_obj = file_obj.Schema()['file'].storage
 		# get the filepath
            	# ATMediaFile (and ATVideo) by default uses ExternalStorage
            	# this file path is the path to the var/files/ ext storage area
            	# inside the zope instance
            	fn_ext_storage = file_ext_storage_obj.getFilepath(file_obj,'file')

		#check the video file actually exists..
		if os.path.getsize(fn_ext_storage) == 0:
			zLOG.LOG('ATMediaFile', zLOG.INFO, ' video object has zero size video file (using external storage object) ... not doing anything ')
			return

		filename_videofile = os.path.basename(fn_ext_storage)

		# get the url to it , via the plone system
            	fn_url = file_obj.absolute_url()

		#prime the data
		self.current_data=dict(	bootstrapURL=fn_url+'/download', 	
					comment='ATVideo object available at %s'%fn_url+'/video_view', 
					filename=filename_videofile,
					file_obj=file_obj
					)
		#do the remote proc call via twisted perspective broker client
		self.useRemoteTorrentServer()
    

    security.declarePublic('useRemoteTorrentServer')
    def useRemoteTorrentServer(self):
 	'''call PBServer to do remote server BitTorrent'''
	(hostname,portnumber) = self.getRemote_bittorrent_server_url().split(":")
        zLOG.LOG('ATMediaFile',zLOG.INFO,"starting pb.PBClientFactory using %s:%s"%(hostname,portnumber))
        client = pb.PBClientFactory()
        ## need to get hostname/port of perspective broker server
	# 5 second timeout
        reactor.connectTCP(hostname, int(portnumber), client, 5)
        dfr = client.getRootObject()
        dfr.addCallbacks(self._gotRemote, self._didFail)
        #start the reactor, without installing sig handlers.
        reactor.run(installSignalHandlers=0)
	#wont get to here until we call self._doStop()
	zLOG.LOG('ATMediaFile',zLOG.INFO,"after reactor.run() in useRemoteTorrentServer")

    security.declarePublic('_gotRemote')
    def _gotRemote( self, remote ):
        try:
            #remote.notifyOnDisconnect(self._didFail)
            #
            # set the correct bootstrap URL etc
	    bootstrapURL = self.current_data['bootstrapURL']
	    filename = self.current_data['filename']
	    custom_comment = self.current_data['comment']
	    #the remote procedure
            zLOG.LOG('ATMediaFile',zLOG.INFO, "_gotRemote %s -- using %s %s %s %s " % (remote, bootstrapURL,  self.getBittorrent_tracker_url(), filename, custom_comment))
            remote_deferred=remote.callRemote('DownloadMakeTorrentAndSeed',
                                { 'bootstrapURL':bootstrapURL,
                                  'trackerURL': self.getBittorrent_tracker_url(),
                                  'filename':filename,
                                  'comment':'torrent auto-generated by ATMediaFile object. http://svn.plone.org/svn/collective/ATMediaFile/branches/engage ' + custom_comment})
	    remote_deferred.addCallbacks(self._remoteTorrentGenSuccess,self._remoteTorrentGenFail)
	    zLOG.LOG('ATMediaFile',zLOG.INFO,"_gotRemote got deferred object from remote call %s " % remote_deferred)
        except:
            traceback.print_exc()

    security.declarePublic('_remoteTorrentGenSuccess')
    def _remoteTorrentGenSuccess( self, foo ):
        try:
            zLOG.LOG('ATMediaFile',zLOG.INFO, "ATMediaFile:remote server called worked!. yay!!\n returned %s " % foo)

	    #create the associated BitTorrent file
	    #foo contains the .torrent file made  on the remote server.  

 	    #this is where the .torrent file ends up , locally, after returned from the remote PB torrent-making server.
	    torrent_filepath = self.current_data['filename'] + ".torrent"
	    #make the id of the bittorrent file object
	    bt_id = os.path.basename(self.current_data['filename']) + ".torrent"
	    bt_container = self.current_data['file_obj'].aq_parent 
	    zLOG.LOG('ATMediaFile',zLOG.INFO,'Using %s as container for BitTorrentFile object' % bt_container.getId())
	    # first delete old BitTorrent File object
	    try:
	    	bt_container.manage_delObjects([bt_id])
	    except:
		pass
	    #try to make the BitTorrent file in the choosen container
            #new_container.invokeFactory('BitTorrentFile',id=id)
	    #do it with unchecked security
	    from Products.CMFPlone.utils import _createObjectByType
	    _createObjectByType('BitTorrentFile',bt_container,bt_id)
	    #get the new object back
            new_obj = getattr(bt_container, bt_id)
	    # set the title of the newly created BitTorrent Object
            new_obj.setTitle(bt_id)
            # inject the content to the file and set the mimetype and filename
            torrent_mimetype = "application/x-bittorrent"
            new_obj.setFile(foo, mimetype=torrent_mimetype, filename=torrent_filepath)
            # hide the bittorrent file in the navTree
            new_obj.setExcludeFromNav(True)
            # relate the video object with its corresponding bittorrent object
            self.current_data['file_obj'].setRelatedItems(new_obj)
            new_obj.reindexObject()

	    #stop the reactor
	    self._doStop()

        except:
            traceback.print_exc()


    security.declarePublic('_remoteTorrentGenFail')
    def _remoteTorrentGenFail( self, foo ):
        try:
            # this is called if the call inside the remote pb server failed
            zLOG.LOG('ATMediaFile',zLOG.INFO, "ATMediaFile:remote server called FAILED. reason is  %s " % foo)
	    #stop the reactor
	    self._doStop()

        except:
            traceback.print_exc()


    security.declarePublic('_didFail')
    def _didFail( self, foo ):
        try:
            # this is called if we didnt connect to the remote pb server
            zLOG.LOG('ATMediaFile',zLOG.INFO, "ATMediaFile:remote server couldnt establish contact. reason is %s " % foo)
	    #stop the reactor
	    self._doStop()
	    
        except:
            traceback.print_exc()
     
    security.declarePublic('_doStop')
    def _doStop(self):
        #this needs to be 'crash' rather than 'stop' since it doesnt seem to actually stop
	#properly (the second time??) when 'stop' is called, and i dont know how to break into the python process(es) to debug it.
    	reactor.crash()
	zLOG.LOG('ATMediaFile',zLOG.INFO,"ATMediaFile:called reactor.stop")
 
    security.declarePublic('getTorrentSupport')
    def getTorrentSupport(self):
        """
         checks if BitTorrent >= 4.4 is installed correctly _locally_
         returns True if support is there
         return False is an error is encountered when trying to access a BitTorrent function
        """
        bittorrentsupport = False
        try:
            from BitTorrent.platform import install_translation
            from BitTorrent.makemetafile import make_meta_files
	    bittorrentsupport = True
        except:
	    zLOG.LOG('ATMediaFile', zLOG.INFO, 'Warning: in getTorrentSupport() - ATMediaFile has no _local server_ BitTorrent support! In order to have BitTorrent support you need the latest ATMediaFile, please update.')
            bittorrentsupport = False
        return bittorrentsupport

    security.declarePublic('getRemoteTorrentSupport')
    def getRemoteTorrentSupport(self):
        """
         checks if Twisted is installed (tested with 2.5)
         returns True if support is there
         return False is an error is encountered when trying to access a BitTorrent function
        """
        remote_bittorrentsupport = False
        try:
    	    from twisted.spread import pb
    	    from twisted.internet import reactor
	    remote_bittorrentsupport = True
	except:
	    zLOG.LOG('ATMediaFile', zLOG.INFO, 'Warning: in getRemoteTorrentSupport() - ATMediaFile has no _remote server_ BitTorrent support! In order to have BitTorrent support you need Twisted 2.5 installed.')
            remote_bittorrentsupport = False
        return remote_bittorrentsupport

    security.declarePublic('createTorrentFile')
    def createTorrentFile(self,file_obj):
        """ Local BitTorrent support.
        create a _local_ BitTorrent file (.torrent) for a given Fileobject
        returns a .torrent file with the name of given Fileobject and stores it
        to the external filestorage directory and creates a Filereference in the ZODB
        for the external file
        """
 
 	#check for local bittorrent support.
        if (self.getEnable_bittorrent() and self.getTorrentSupport() ):
	    zLOG.LOG('ATMediaFile',zLOG.INFO,'Starting createTorrentFile')
            # get the ExternalStorage object for the video file
            file_ext_storage_obj = file_obj.Schema()['file'].storage
            # get the filepath
            # ATMediaFile (and ATVideo) by default uses ExternalStorage
            # this file path is the path to the var/files/ ext storage area
            # inside the zope instance
            fn_ext_storage = file_ext_storage_obj.getFilepath(file_obj,'file')

            # the path for PloneTramline
            # handles large files in a separate area altogether from zope
            try:
                from Products.PloneTramline.filefield_patch import REPOSITORY_PATH
                #its still in upload, not repository, because we havent signalled back to the 
                #the output mod_python (tramline) filter!
                fn_tramline = REPOSITORY_PATH + "/upload/" + str(self.Schema()['file'].get(self))
                using_tramline = True
            except:
                using_tramline = False
                fn_tramline = None
    

            # get the url to it , via the plone system
            fn_url = file_ext_storage_obj.getInstancePath(file_obj)

	    if using_tramline:
                fn = fn_tramline
            else:
                fn = fn_ext_storage

	   	    
            # add the ".torrent" into plone as a File object
	    # wih the actual ".torrent" file attached to this object
	
            torrent_filepath = fn + ".torrent"

	    #check that the _actual_ object file has a filesize > 0
	    # if it does have a zero size, dont make anything !! no .torrents, and no BitTorrentFile objects!!
	    if os.path.getsize(fn) == 0:
	    	return

	    zLOG.LOG('ATMediaFile',zLOG.INFO,'Making .torrent using path %s'%torrent_filepath)
            # now make the .torrent file
            # see /usr/lib/python2.4/BitTorrent/makemetafile.py for the api
	    torrent_comment = "torrent made by ATMediaFile product from http://svn.plone.org/view/collective/ATMediaFile"
            make_meta_files( self.getBittorrent_tracker_url(), [fn], piece_len_pow2=18, comment=torrent_comment, target = torrent_filepath)
    
            #import pdb; pdb.set_trace()
            container = self
            # plone id (title of Content)
            # name of the video filename + ".torrent"
            id = os.path.basename(fn) + ".torrent"

            # read content of generated torrent file
            # hint: read file BEFORE invokeFactory (externalStorage) empties the file at creating :-)
            # rename the file on the filesystem, create a external storage object with the same filename,
            # with dummycontent, then overwrite the dummy file on the filesystem with the original file
            os.rename(torrent_filepath, torrent_filepath + ".tmp")
                
            # create a new container with the object/folder containing the videoobject
            # so the .torrent gets stored in the same relative position as the videofile
            new_container = file_obj.aq_parent
	    
	    zLOG.LOG('ATMediaFile',zLOG.INFO,'Using %s as container for BitTorrentFile object' % new_container.getId())
	    # first delete old BitTorrent File object
	    try:
	    	new_container.manage_delObjects([id])
	    except:
		pass
	    #try to make the BitTorrent file in the choosen container
            #new_container.invokeFactory('BitTorrentFile',id=id)
	    #do it with unchecked security
	    from Products.CMFPlone.utils import _createObjectByType
	    _createObjectByType('BitTorrentFile',new_container,id)
	    #get the new object back
            new_obj = getattr(new_container, id)

            # set the title of the newly created BitTorrent Object
            new_obj.setTitle(id)
            # inject the dummy content to the file and set the mimetype and filename

            torrent_mimetype = "application/x-bittorrent"
            new_obj.setFile("--dummy--", mimetype=torrent_mimetype, filename=torrent_filepath)
            # overwrite the dummy file on the fileystem with the original file
	    # get the filepath of the dummyfile , since it could be anywhere, since we change containers above sometimes
	    # get the new 'dummy' torrent filepath
	    dummy_torrent_fp = new_obj.schema['file'].storage.getFilepath(new_obj,'file')
            os.remove(dummy_torrent_fp)
	    zLOG.LOG('ATMediaFile',zLOG.INFO,'torrent path %s :: bittorrent file path %s ' % (torrent_filepath , dummy_torrent_fp))
            os.rename(torrent_filepath + ".tmp", dummy_torrent_fp)
            
            # hide the bittorrent file in the navTree
            new_obj.setExcludeFromNav(True)
               
            # relate the video object with its corresponding bittorrent object
            file_obj.setRelatedItems(new_obj)
    
            new_obj.reindexObject()
    

registerType(ATMediaFileTool, PROJECTNAME)
# end of class ATMediaFileTool

##code-section module-footer #fill in your manual code here
##/code-section module-footer



