#standard imports
from StringIO import StringIO
from Products.CMFCore.utils import getToolByName
#thirdparty imports
from Products.ATVocabularyManager.config import TOOL_NAME as ATVOCABULARYTOOL
from Products.ATCountryWidget.CountryTool import CountryUtils, Country
from Products.CMFCore.WorkflowCore import WorkflowException
#from Products.fatsyndication.config import TOOL_ID as SYN_PORTAL_TOOL_NAME

#ATVideo config imports
from Products.ATVideo.config import TOPLEVEL_FOLDER, GENRE_FOLDER, CATEGORIES_FOLDER, SYN_MANAGER, COUNTRIES_FOLDER, DEFAULT_MB_FLDR, USE_SYN_MANAGER, SUBMISSIONS_FOLDER


def addCatalogKeys(self, out):

    pc = getToolByName(self, 'portal_catalog')
    refresh = False
    index = pc.indexes()
    metadata = pc.schema()

    # Things to add to the catalog.
    keys = (dict(name = 'isPublishableATEngageVideoObj',
                 meta = True,
                 idx  = False),)

    # Iter to add the things where they belong.
    for key in keys:

        name = key['name']
        if key['idx'] and name not in index:
            pc.addIndex(name, 'FieldIndex')
            out.write('Added %s index to portal_catalog\n' % name)
            refresh = True

        if key['meta'] and name not in metadata:
            out.write('Added %s metadata column to portal_catalog\n' % name)
            pc.addColumn(name)
            refresh = True

    if refresh is True:
        pc.refreshCatalog()

    return refresh



def publishObject(self, obj,out):
   #XXX  you could change ownership here.
   #username = 'Plumi'
   #obj.setCreators([username])
   #self.plone_utils.changeOwnershipOf(obj, username, 1)
   #publish it straight away
   try:
        workflow = getToolByName(self,'portal_workflow')
        workflow.doActionFor(obj,'publish')
   except WorkflowException:
   	out.write("caught an exception in publishObject with object %s " % obj)
        pass

def configureVPIP_js(self,out):
    """ adds VPIP javascript
    """
    from Products.ResourceRegistries.config import CSSTOOLNAME, JSTOOLNAME
    jstool = getToolByName(self, JSTOOLNAME)
    #blank condition - always available
    CONDITION=''
    for id in ('vpip.js','jquery.js'):
    	jstool.manage_removeScript(id=id)
    # for some reason jquery.js breaks when compressed, so we do the installs separately
    # vpip.js can be compressed
    jstool.manage_addScript(id='vpip.js',
            expression=CONDITION,
            enabled=True,
	    compression='safe',
            cookable=True)
    # jquery.js can not be compressed
    jstool.manage_addScript(id='jquery.js',
            expression=CONDITION,
            enabled=True,
            compression='none',
            cookable=True) 

def configATEngageVideo(self,out):
    """
    	switch off ATVideo, ATEngageVideo, BitTorrentFile in portal_types
	Change the title of ATEngageVideo to 'Video', ATEngageVideoFolder to 'Video Folder'

    """
    types = self.portal_types
    atvideo = getattr(types, 'ATVideo',None)
    if atvideo is not None:
        out.write("ATVideo detected, removing Implicitly Addable\n")
        types.ATVideo.manage_changeProperties(global_allow=0)
    else:
        out.write("ATVideo implicitly addable not removed, ATVideo not detected in portal_types\n")
    out.write("ATVideo is not implicitly addable\n")

    atengagevideo = getattr(types, 'ATEngageVideo',None)
    if atengagevideo is not None:
        out.write("Changing ATEngageVideo portal_type title to 'Video', removing implicitly addable\n")
        types.ATEngageVideo.manage_changeProperties(title='Video')
	types.ATEngageVideo.manage_changeProperties(global_allow=0)

    else:
        out.write("ATEngageVideo title in portal types not changed, no ATEngageVideo Type")
    
    #needs to be addable , so when new users join we can add these types of folders to their home folder.
    atengagevideofolder = getattr(types, 'ATEngageVideoFolder',None)
    if atengagevideofolder is not None:
	types.ATEngageVideoFolder.manage_changeProperties(title='Video Folder')
    else:
        out.write("ATEngageVideoFolder title in portal types not changed, no ATEngageVideoFolder Type")

    btfile = getattr(types, 'BitTorrentFile',None)
    if btfile is not None:
        out.write("BitTorrentFile detected, removing Implicitly Addable\n")
        types.BitTorrentFile.manage_changeProperties(global_allow=0)
    else:
        out.write("BitTorrentFile implicitly addable not removed, BitTorrentFile not detected in portal_types\n")
    out.write("BitTorrentFile is not implicitly addable\n")

    #Callout content type, only to be added in CalloutFolder
    callout = getattr(types, 'Call Out',None)
    if callout is not None:
	types['Call Out'].manage_changeProperties(global_allow=0)
    else:
        out.write("Callout not installed!")
    out.write("Callout is not implicitly addable\n")
    #XXX need to stop ability to add PressRoom, PressContact, PressRelease , PressClip


    #hide member folder from nav
    # add to list of ids to make invisible
    portalProperties = getToolByName(self, 'portal_properties', None)
    if portalProperties is not None:
        navtreeProperties = getattr(portalProperties, 'navtree_properties', None)
        if navtreeProperties:
            navtreeProperties.idsNotToList = list(navtreeProperties.idsNotToList) + \
                                  [toolname for toolname in [DEFAULT_MB_FLDR] \
                                            if toolname not in navtreeProperties.idsNotToList]


    ## exclude certain items from navigation
    #hide tools in the navigation
    #meta types!!
    if USE_SYN_MANAGER:
    	for t in ['SyndicationManager','SyndicationFeed']:
        	try:
            		if t not in self.portal_properties.navtree_properties.metaTypesNotToList:
                		self.portal_properties.navtree_properties.metaTypesNotToList =  list(self.portal_properties.navtree_properties.metaTypesNotToList) + [t]
        	except TypeError, e:
            		out.write("Attention: could not set the navtree properties:%s"%e)

def changeWorkflowForCustomFolders(self, out):
   """changes to folder_workflow"""

   from Products.ATContentTypes.config import WORKFLOW_FOLDER
   workflowTool = self.portal_workflow    
   workflowTool.setChainForPortalTypes(('ATEngageVideoFolder',), WORKFLOW_FOLDER)
   workflowTool.setChainForPortalTypes(('Callout Folder',), WORKFLOW_FOLDER)
   #workflowTool.updateRoleMappings()

def useViewAction(self, out):
    """use view action in listings"""
    props = self.portal_properties.site_properties
    
    useView = getattr(props, 'typesUseViewActionInListings', None)
    if useView is not None:
        if 'ATVideo' not in useView:
            props._updateProperty('typesUseViewActionInListings', list(useView) + ['ATVideo']) 
	if 'ATEngageVideo' not in useView:
            props._updateProperty('typesUseViewActionInListings', list(useView) + ['ATEngageVideo'])

def addCTRExtensions(self, out):
    """
    add predicates for 'ATVideo' file extensions to the content type registry
    """
    ## make this configurable ???
    predicate_id = 'video_ext'
    extensions = 'mov, wmv, rm, swf, flv, mpg'  # comma separated list
    #rm, .wmv, .swf, .flv, .mov or .mpg
    type_name = 'ATEngageVideo'
    
    ctr = self.content_type_registry
    predicate_ids = [p[0] for p in ctr.listPredicates()]
    if predicate_id not in predicate_ids:
        ctr.addPredicate(predicate_id, 'extension')
        out.write("Adding a '%s' predicate to the content type registry\n"\
                  % predicate_id)
    predicate = ctr.getPredicate(predicate_id)
    predicate.edit(extensions)
    ctr.updatePredicate(predicate_id, predicate, type_name)
    ctr.reorderPredicate(predicate_id, 0)
    out.write("'%s' Predicates installed.\n" % extensions)
    
def addCTRMimetypes(self, out):
    """
    add a predicate for 'ATVideo' mime type to the content type registry
    """
    predicate_id = 'video'
    mimetype = 'video'
    type_name = 'ATEngageVideo'

    ctr = self.content_type_registry
 
    if ctr.getPredicate(predicate_id):
        out.write("Adding a '%s' predicate to the content type registry\n"\
                  % predicate_id)
        ctr.removePredicate(predicate_id)
    ctr.addPredicate(mimetype, 'major_minor')
    ctr.getPredicate(predicate_id).edit(major=mimetype, minor='')
    ctr.assignTypeName(predicate_id, type_name)
    ctr.reorderPredicate(predicate_id, 1)
    out.write("Predicate for MIME type '%s' installed.\n" % mimetype)

def extendMemberdata(self, video_genres_list, out):
    pm = self.portal_memberdata
    if not pm.hasProperty('city'):
        pm.manage_addProperty('city', '', 'string')
    if not pm.hasProperty('street'):
        pm.manage_addProperty('street', '', 'string')
    if not pm.hasProperty('postcode'):
        pm.manage_addProperty('postcode', '', 'string')
    if not pm.hasProperty('phone'):
        pm.manage_addProperty('phone', '', 'string')
    if not pm.hasProperty('url'):
        pm.manage_addProperty('url', '', 'string')
    if not pm.hasProperty('genre_interests_selection'):
        #pm.manage_addProperty('genre_interests_selection', ('Documentary','Music','Experimental','Animation','Fiction',), 'lines')
	pm.manage_addProperty('genre_interests_selection',video_genres_list, 'lines')
    if not pm.hasProperty('userbio'):
        pm.manage_addProperty('userbio', '', 'string')

    if not pm.hasProperty('genre_interests'):
        pm.manage_addProperty('genre_interests', 'genre_interests_selection', 'multiple selection')
    if not pm.hasProperty('activities_selection'):
        pm.manage_addProperty('activities_selection', ('Production','Distribution','Screening',), 'lines')
    if not pm.hasProperty('activities'):
        pm.manage_addProperty('activities', 'activities_selection', 'multiple selection')
    if not pm.hasProperty('media_formats_selection'):
        pm.manage_addProperty('media_formats_selection', ('TV','DVD','Web','Film',), 'lines')
    if not pm.hasProperty('media_formats'):
        pm.manage_addProperty('media_formats', 'media_formats_selection', 'multiple selection')
    out.write("Additional Members Data installed.\n")

def addVideoVocabs(self, out, vocabs):
    """
    	adding the vocabs to ATVocabularyManager
    """

    portal=getToolByName(self,'portal_url').getPortalObject()
    atvm = getToolByName(portal, ATVOCABULARYTOOL)

    for vkey in vocabs.keys():
            # create vocabulary if it doesnt exist:
            vocabname = vkey
            if not hasattr(atvm, vocabname):
                print >>out, "adding vocabulary %s" % vocabname
                atvm.invokeFactory('SimpleVocabulary', vocabname)
            vocab = atvm[vocabname]
	    #delete the 'default' item
	    if hasattr(vocab, 'default'):
	    	vocab.manage_delObjects(['default'])

            for (ikey, value) in vocabs [vkey]:
                if not hasattr(vocab, ikey):
                    vocab.invokeFactory('SimpleVocabularyTerm', ikey)
                    vocab[ikey].setTitle(value)

	    ## HACK for video_genre
	    if vocabname == 'video_genre':
	    	#add a 'genre' of (default_genre_term_id,u'-') , which in a selection widget , will be the 'none' option..
	        #calling it 'aa-none' means it gets sorted first (by our monkey patch of NamedVocabulary.getDisplayList which sorted by key)
		#and will be the 'default' selection in a single valued selection widget
		default_genre_term_id = 'aaa_none'
		if not hasattr(vocab,default_genre_term_id):
			vocab.invokeFactory('SimpleVocabularyTerm',default_genre_term_id)
			vocab[default_genre_term_id].setTitle(u'-')


def createSmartSearchFolders(self, out, vocabs):
        """ Create smart search capabilities
        
        Things done here:
            - Creates new indexes (genre, categories)
            - Binds indexes to smart folders
            - Creates a smart folder object for these searches
	    adapted from http://plone.org/documentation/how-to/adding-new-fields-to-smart-folders-search
        
        """ 
        
        # dummy class used with manage_addIndex
	# XXX I dont know what this is for.
        class args:
            def __init__(self, **kw):
                self.__dict__.update(kw)
            def keys(self):
                return self.__dict__.keys()
            
        # Tune up catalog tool
        
        catalog_tool = self.portal_catalog
        
	# Clean up previous install if there is some old stuff
        try:
            catalog_tool._removeIndex("getGenre")
        except:
            pass
    
        try:
            catalog_tool._removeIndex("getCategories")
        except:
            pass    
        try:
            catalog_tool._removeIndex("getCountries")
        except:
            pass    
        try:
            catalog_tool._removeIndex("getFirstPublishedTransitionTime")
        except:
            pass    


        try:
            catalog_tool.delColumn("getGenre")
        except:
            pass        
        
        try:
            catalog_tool.delColumn("getCategories")
        except:
            pass        
        try:
            catalog_tool.delColumn("getCountries")
        except:
            pass        
        try:
            catalog_tool.delColumn("getFirstPublishedTransitionTime")
        except:
            pass        


        #XXX this is black magic to me 
        extra = args(doc_attr='SearchableText',
                             lexicon_id='plone_lexicon',
                             index_type='Okapi BM25 Rank')    
        
        catalog_tool.manage_addIndex("getGenre", "KeywordIndex", extra)
        catalog_tool.manage_addIndex("getCategories", "KeywordIndex",extra)
	catalog_tool.manage_addIndex("getCountries", "KeywordIndex",extra)
	catalog_tool.manage_addIndex("getFirstPublishedTransitionTime", "DateIndex",extra)

        catalog_tool.manage_addColumn("getGenre")
        catalog_tool.manage_addColumn("getCategories")
	catalog_tool.manage_addColumn("getCountries")
	catalog_tool.manage_addColumn("getFirstPublishedTransitionTime")

        # Update Smart Folder settings           
        smart_folder_tool = self.portal_atct
        
        # Remove existing indexes if there are some
        try:
            smart_folder_tool.removeIndex("getGenre")
        except:
            pass
        
        try:
            smart_folder_tool.removeMetadata("getGenre")
        except:
            pass

        try:
            smart_folder_tool.removeIndex("getCategories")
        except:
            pass

        try:
            smart_folder_tool.removeMetadata("getCategories")
        except:
            pass

        try:
            smart_folder_tool.removeIndex("getCountries")
        except:
            pass

        try:
            smart_folder_tool.removeMetadata("getCountries")
        except:
            pass
 
        try:
            smart_folder_tool.removeIndex("getFirstPublishedTransitionTime")
        except:
            pass

        try:
            smart_folder_tool.removeMetadata("getFirstPublishedTransitionTime")
        except:
            pass

      
        smart_folder_tool.addIndex("getGenre", "Video Genre", "The genre of a video", enabled=True)
        smart_folder_tool.addIndex("getCategories", "Categories", "The categories available on a content type", enabled=True)
	smart_folder_tool.addIndex("getCountries", "Countries", "The countries", enabled=True)
	smart_folder_tool.addIndex("getFirstPublishedTransitionTime", "First published time", "The date and time the video object was first published.", enabled=True)

        smart_folder_tool.addMetadata("getGenre", "Video Genre", "The genre of a video", enabled=True)
        smart_folder_tool.addMetadata("getCategories", "Categories", "The categories available on a content type", enabled=True)
	smart_folder_tool.addMetadata("getCountries", "Countries", "The countries", enabled=True)
	smart_folder_tool.addMetadata("getFirstPublishedTransitionTime", "First published time", "The date and time the video object was first published.", enabled=True)

        # Create Video genre+categories smart folders                     
	# and syndication manager
	#

        try:
            # Remove existing instance if there is one
            self.manage_delObjects([TOPLEVEL_FOLDER])
	except:
	    pass

	# make the folder of smartfolders
    	self.invokeFactory('Folder',id=TOPLEVEL_FOLDER,title="Browse Videos")
	#get the folder object
	toplevel_fldr = getattr (self, TOPLEVEL_FOLDER)
	toplevel_fldr.setExcludeFromNav(True)
	publishObject(self,toplevel_fldr,out)
	#toplevel_fldr.reindexObject()

	# make the syndication manager
	# XXX Not used (03/04/06) , but leave in, if we got back to using FATSyndication
	if USE_SYN_MANAGER:
		syn_portal_tool = getToolByName(self, SYN_PORTAL_TOOL_NAME)
		try:
	    		# Remove existing instance of syndication manager
            		syn_portal_tool.manage_delObjects([SYN_MANAGER])
        	except:
            		pass  
		syn_obj = syn_portal_tool.returnSyndicationManagerFor(toplevel_fldr, id=SYN_MANAGER, title='Syndication Manager for All Categories and Genres')

        # Create smart folders inside folder of categories+genres+videos
	#
       	try:
       		# Remove existing instance if there is one
       		self.manage_delObjects([GENRE_FOLDER, CATEGORIES_FOLDER, COUNTRIES_FOLDER, SUBMISSIONS_FOLDER])
       	except:
       		pass     	

	#add the layers of hierarchy
	# genres and categories and countries
	toplevel_fldr.invokeFactory('Folder',id=GENRE_FOLDER, title='Video Genres')
	genre_fldr = getattr(toplevel_fldr, GENRE_FOLDER)
	publishObject(self,genre_fldr,out)
	
	toplevel_fldr.invokeFactory('Folder',id=CATEGORIES_FOLDER, title='Video Topics')
	categ_fldr = getattr(toplevel_fldr, CATEGORIES_FOLDER)
	publishObject(self,categ_fldr,out)

	toplevel_fldr.invokeFactory('Folder',id=COUNTRIES_FOLDER, title='Countries')
	countries_fldr = getattr(toplevel_fldr, COUNTRIES_FOLDER)
	publishObject(self,countries_fldr,out)

	#toplevel_fldr.invokeFactory('Folder',id=PEOPLE_FOLDER, title='People')
	#users_fldr = getattr(toplevel_fldr, PEOPLE_FOLDER)
	
	toplevel_fldr.invokeFactory('Folder',id=SUBMISSIONS_FOLDER, title='Call Outs')
	submissions_fldr = getattr(toplevel_fldr, SUBMISSIONS_FOLDER)
	publishObject(self,submissions_fldr,out)


	topic_description_string = 'Channel for %s on ' + '%s' % self.Title() 

	for vocab in vocabs['video_genre']:
		new_smart_fldr_id = vocab[0]
	       	try:
            		# Remove existing instance if there is one
            		genre_fldr.manage_delObjects([new_smart_fldr_id])
        	except:
            		pass     	
		#make the new SmartFolder
		genre_fldr.invokeFactory('Topic', id=new_smart_fldr_id,title=vocab[1], description=topic_description_string % vocab[1])
		fldr = getattr(genre_fldr,new_smart_fldr_id)

       		# Filter results to ATEngageVideo
       		type_criterion = fldr.addCriterion('Type', 'ATPortalTypeCriterion' )
		#Have to use the name of the Title (and ATEngageVideo will be re-named by configATEngageVideo to Video!)
		#this will actually use ALL objects with title 'Video', which means atm, ATEngageVideo and ATVideo
       		type_criterion.setValue("Video")

		# Filter results to this individual genre
       		type_criterion = fldr.addCriterion('getGenre', 'ATSimpleStringCriterion' )
		#
		#match against the ID of the vocab term. see getGenre in engage.py (ATEngageVideo object)
       		type_criterion.setValue(vocab[0])


		## add criteria for showing only published videos
		state_crit = fldr.addCriterion('review_state', 'ATSimpleStringCriterion') 
		state_crit.setValue('published')

      		#sort on reverse date order, using the first published time transition
        	sort_crit = fldr.addCriterion('getFirstPublishedTransitionTime',"ATSortCriterion")
        	sort_crit.setReversed(True)

		#enable syndication in folder (via qRSS2Syndication product)
		enableqRSS2(self, fldr, out)

		if USE_SYN_MANAGER:
			# make a referenced feed object from FATSyndication
			# make the newly made smartfolder the referenced object
			feed_id = vocab[0]
			try:
            			# Remove existing instance if there is one
            			self.manage_delObjects([feed_id])
        		except:
            			pass 
			syn_obj.invokeFactory('SyndicationFeed', id=feed_id, title=vocab[1])
			syn_feed = getattr(syn_obj, feed_id)	
     			syn_feed.addReferencedFeedSource(fldr) 
			# TODO
			# exclude it from the navigation
			# change the ReferencedFeedSource id to something nicer than a random int (check fatsyndication/feed.py)

		#fldr.reindexObject()	

       	for vocab in vocabs['video_categories']:
    		new_smart_fldr_id = vocab[0]
	       	try:
            		# Remove existing instance if there is one
            		categ_fldr.manage_delObjects([new_smart_fldr_id])
        	except:
            		pass     	
		#make the new SmartFolder
		categ_fldr.invokeFactory('Topic', id=new_smart_fldr_id,title=vocab[1], description=topic_description_string % vocab[1])
		fldr = getattr(categ_fldr,new_smart_fldr_id)

       		# Filter results to ATEngageVideo
       		type_criterion = fldr.addCriterion('Type', 'ATPortalTypeCriterion' )
       		type_criterion.setValue("Video")

		# Filter results to this individual category
       		type_criterion = fldr.addCriterion('getCategories', 'ATListCriterion' )
		#
		#match against the ID of the vocab term. see getCategories in engage.py (ATEngageVideo object)
       		type_criterion.setValue(vocab[0])
		#match if any vocab term is present in the video's selected categories
		type_criterion.setOperator('or')
	
		## add criteria for showing only published videos
		state_crit = fldr.addCriterion('review_state', 'ATSimpleStringCriterion') 
		state_crit.setValue('published')
      		#sort on reverse date order
        	sort_crit = fldr.addCriterion('getFirstPublishedTransitionTime',"ATSortCriterion")
        	sort_crit.setReversed(True)


		#enable syndication in folder (via qRSS2Syndication product)
		enableqRSS2(self, fldr, out)

		if USE_SYN_MANAGER:	
			# make a referenced feed object from FATSyndication
			# make the newly made smartfolder the referenced object
			feed_id = vocab[0]
			try:
            			# Remove existing instance if there is one
            			self.manage_delObjects([feed_id])
        		except:
            			pass 
			syn_obj.invokeFactory('SyndicationFeed', id=feed_id, title=vocab[1])
			syn_feed = getattr(syn_obj, feed_id)	
     			syn_feed.addReferencedFeedSource(fldr) 
			# TODO
			# exclude it from the navigation
			# change the ReferencedFeedSource id to something nicer than a random int (check fatsyndication/feed.py)

		#fldr.reindexObject()


	#get the countries from the countrytool!
	# nb: this means that the setup method for the countries should be called BEFORE
	# this one
	countrytool = getToolByName(self,CountryUtils.id)	
	cdict = list()
 	for area in countrytool._area_list:
            for country in area.countries:
                cdict.append([country.isocc,country.name])

	for country in cdict:
    		new_smart_fldr_id = country[0]
	       	try:
            		# Remove existing instance if there is one
            		countries_fldr.manage_delObjects([new_smart_fldr_id])
        	except:
            		pass     	
		#make the new SmartFolder
		countries_fldr.invokeFactory('Topic', id=new_smart_fldr_id,title=country[1],description=topic_description_string % country[1])

		fldr = getattr(countries_fldr,new_smart_fldr_id)

       		# Filter results to ATEngageVideo
       		type_criterion = fldr.addCriterion('Type', 'ATPortalTypeCriterion' )
       		type_criterion.setValue("Video")

		# Filter results to this individual category
       		type_criterion = fldr.addCriterion('getCountries', 'ATListCriterion' )
		#
		#match against the ID of the vocab term. see getCategories in engage.py (ATEngageVideo object)
       		type_criterion.setValue(country[0])
		#match if any vocab term is present in the video's selected categories
		type_criterion.setOperator('or')

		## add criteria for showing only published videos
		state_crit = fldr.addCriterion('review_state', 'ATSimpleStringCriterion') 
		state_crit.setValue('published')
      		#sort on reverse date order
        	sort_crit = fldr.addCriterion('getFirstPublishedTransitionTime',"ATSortCriterion")
        	sort_crit.setReversed(True)


		#enable syndication in folder (via qRSS2Syndication product)
		enableqRSS2(self, fldr, out)
		
		#fldr.reindexObject()
	#
	# Callouts smart folders
	#
	topic_description_string = "CallOuts for Topic - %s "
       	for submission_categ in vocabs['submission_categories']:
    		new_smart_fldr_id = submission_categ[0]
	       	try:
            		# Remove existing instance if there is one
            		submissions_fldr.manage_delObjects([new_smart_fldr_id])
        	except:
            		pass     	
		#make the new SmartFolder
		submissions_fldr.invokeFactory('Topic', id=new_smart_fldr_id,title=submission_categ[1], description=topic_description_string % submission_categ[1])
		fldr = getattr(submissions_fldr,new_smart_fldr_id)

       		# Filter results to Callouts
       		type_criterion = fldr.addCriterion('Type', 'ATPortalTypeCriterion' )
		#the title of the type, not the class name, or portal_type 
       		type_criterion.setValue("Call Out")

		# Filter results to this individual category
       		type_criterion = fldr.addCriterion('getCategories', 'ATListCriterion' )
		#
		#match against the ID of the vocab term. see getCategories in callout.py (Callout object)
       		type_criterion.setValue(submission_categ[0])
		#match if any vocab term is present in the video's selected categories
		type_criterion.setOperator('or')
	
		## add criteria for showing only published videos
		state_crit = fldr.addCriterion('review_state', 'ATSimpleStringCriterion') 
		state_crit.setValue('published')
      		#sort on reverse date order
        	sort_crit = fldr.addCriterion('modified',"ATSortCriterion")
        	sort_crit.setReversed(True)

		##XXX make it only if the Callout hasnt closed.

		#enable syndication in folder (via qRSS2Syndication product)
		enableqRSS2(self, fldr, out)

		if USE_SYN_MANAGER:	
			# make a referenced feed object from FATSyndication
			# make the newly made smartfolder the referenced object
			feed_id = submission_categ[0]
			try:
            			# Remove existing instance if there is one
            			self.manage_delObjects([feed_id])
        		except:
            			pass 
			syn_obj.invokeFactory('SyndicationFeed', id=feed_id, title=submission_categ[1])
			syn_feed = getattr(syn_obj, feed_id)	
     			syn_feed.addReferencedFeedSource(fldr) 
			# TODO
			# exclude it from the navigation
			# change the ReferencedFeedSource id to something nicer than a random int (check fatsyndication/feed.py)

		#fldr.reindexObject()


	#update the catalogue, to reindex any existing videos, so that they appear in these newly rceated smart folders
	#self.portal_catalog.refreshCatalog()
        out.write("Created video categories, genre, countries, and callfor submissions categories smart folders\n")

def addCustomAreasATCountryWidget(self,out):
    countrytool = getToolByName(self, CountryUtils.id)
    #
    #reset the country tool
    countrytool.manage_countries_reset()

    #Add countries missing from the standard installs
    # 
    countrytool.manage_countries_addCountry('WP','West Papua')
    #XXX this is a state of America, not a country - but thats just politics!!
    countrytool.manage_countries_addCountry('HA','Hawaii')
    countrytool.manage_countries_addCountry('BU','Bougainville')

    #update three more pre-exisiting countries 
    #change Myanmar to Burma
    #Lao Peoples blah blah to just Laos
    #Viet Nam to Vietnam
    countries = countrytool._country_list
    countries[countries.index(Country('MM'))].name="Burma"
    countries[countries.index(Country('LA'))].name="Laos"
    countries[countries.index(Country('VN'))].name="Vietnam"
    countries[countries.index(Country('NZ'))].name="NZ-Aotearoa"

    #add our areas
    countrytool.manage_countries_addArea('South East Asia')
    countrytool.manage_countries_addCountryToArea('South East Asia', ['SG','TH','VN','ID','PH','LA','MY','KH','BN','MM'])
    countrytool.manage_countries_sortArea('South East Asia')

    countrytool.manage_countries_addArea('Oceania') 
    countrytool.manage_countries_addCountryToArea('Oceania', ['AU','NZ','PG','TL','WP','FJ','SB','HA','NC','VU','WS','BU','NR','TO','TV','GU','KI','FM','PF','MH','MP','PW','PN','TK','AQ'])
    countrytool.manage_countries_sortArea('Oceania')

def enableqRSS2(self,folder,out):
    #from Products.qRSS2Syndication.utils import setupRSS2Types

    #publish the folder first
    publishObject(self,folder,out)

    out.write('doing qRSS2 setup with %s\n' % folder.Title())
    portal_syn = getToolByName(self,'portal_syndication',None)
    try:
    	portal_syn.enableSyndication(folder)
    except:
	pass
    syInfo =  getattr(folder, 'syndication_information', None)
    types2Syn=['ATVideo','ATEngageVideo','Call Out']
    only_published = 1
    include_subfolders = 1
    articles_number = 20
    enable_itunes = 1
    itunes_image_url = ''
    itunes_explicit = 0
    if syInfo is not None:
	out.write('doing actual setup with syndication_information object\n')
 	syInfo.rss2_types = list(types2Syn)
    	syInfo.only_published = only_published
    	syInfo.include_subfolders = include_subfolders
    	syInfo.max_items = articles_number
    	syInfo.enable_itunes = enable_itunes
    	syInfo.itunes_image_url = itunes_image_url
    	syInfo.itunes_explicit = itunes_explicit	
    out.write('syndication rss2 setup done.\n')

def editUserActions(self, out):
    """
    Adds link to author page in the users action bar
    """
    #pm = getToolByName(self,'portal_membership')

    #XXX seems to be an overlap now?
    # whats the go ?
    # andy 15/6/2006

    #pm.addAction('authorpage'
#	,'Author Page'
#	,'python: portal.portal_url.getPortalObject().absolute_url()+"/author/"+member.getUserName()'
#	,'python: member and portal.portal_membership.getHomeFolder() is not None'
#	,'View'
#	,'user'
#	, visible=1)

#    out.write("Created Author page link in the members action bar.\n")


def kupu_refs(self):
    tool = getToolByName(self, 'kupu_library_tool')
    refs = { 'tool':tool,
             'linkables':tool._res_types['linkable'],
             'medias':tool._res_types['mediaobject'],
             'collections':tool._res_types['collection'] }
    return refs

def configure_kupu(self, out):
    KUPU_LINKABLES = ('ATEngageVideo','Call Out')
    KUPU_MEDIAS = ('')
    KUPU_COLLECTIONS = ('ATEngageVideoFolder','Callout Folder','Topic')
    refs = kupu_refs(self)
    tool = refs['tool']

    linkables = list(refs['linkables'])
    medias = list(refs['medias'])
    collections = list(refs['collections'])

    for t in KUPU_LINKABLES:
        if t not in linkables:
            linkables.append(t)

    for t in KUPU_COLLECTIONS:
        if t not in collections:
            collections.append(t)

    tool._res_types.update({'linkable':tuple(linkables),
                            'media':tuple(medias),
                            'collection':tuple(collections)})

    print >> out, "Added ATEngageVideo and ATEngageVideoFolder configuration to Kupu Library Tool"

def install(self):
    """
    Install method for ATVideo
    """
    out = StringIO()

    vocabs = {}

    vocabs['video_categories'] = (
	    ('poverty', u'Poverty / Development'),
	    ('indigenous', u'Indigenous'),
	    ('refugee', u'Refugee / Migration'),
	    ('health', u'Health'),
	    ('corporations', u'Corporations / Privatisation'),
	    ('globalisation', u'Globalisation'),
	    ('law', u'Law / Justice'),
	    ('work', u'Work'),
	    ('consumerism', u'Consumerism'),
	    ('war', u'War / Peace'),
	    ('human', u'Human Rights'),
	    ('disability', u'Disability Rights'),
	    ('gender', u'Gender / Sexuality'),
	    ('race', u'Race'),
	    ('religion', u'Religion'),
	    ('art', u'Art / Culture'),
	    ('internet', u'Internet'),
	    ('media', u'Media'),
	    ('activism', u'Activism'),
	    ('politics', u'Politics'),
	    ('education', u'Education'),
	    ('biodiversity', u'Biodiversity'),
	    ('climate', u'Climate Change'),
	    ('conservation', u'Forests / Conservation'),
	    ('nuclear', u'Nuclear'),
	    ('sustainablity', u'Sustainability'),
	    ('animal', u'Animal Rights'),
	    ('water', u'Water'),
	    ('biotech', u'Biotech'),
	    ('civillib',u'Civil Liberties'),
    )
    vocabs['video_genre'] = (
            ('documentary', u'Documentary'),
	    ('experimental', u'Experimental'),
	    ('fiction', u'Fiction'),
	    ('animation', u'Animation'),
	    ('music', u'Music'),
	    ('news', u'News Report'),
    )
    # XXX this is currently ununsed - perhaps it will later
    vocabs['video_language'] = (
            ('english', u'English'),
	    ('german', u'German'),
	    ('italian', u'Italian'),
    )
    vocabs['submission_categories'] = (
	    ('festival', u'Festival'),
	    ('screening', u'Screening'),
	    ('dvd', u'DVD'),
	    ('production', u'Production'),
	    ('conference', u'Conference'),
	    ('workshop', u'Workshop'),
	    ('crew', u'Crew'),
	    ('competition', u'Competition'),
	    ('other', u'Other'),

    )

    useViewAction(self, out)
    addCTRExtensions(self, out)
    addCTRMimetypes(self, out)
    extendMemberdata(self, [ x[0] for x in vocabs['video_genre'] ], out)
    addVideoVocabs(self, out, vocabs)
    addCustomAreasATCountryWidget(self, out)
    createSmartSearchFolders(self, out, vocabs)
    editUserActions(self, out)
    #now disable the types we dont want
    #users to be able to add
    configATEngageVideo(self, out) 
    changeWorkflowForCustomFolders(self,out) 
    configure_kupu(self, out)
    configureVPIP_js(self,out)

    # install new catalog indexes/metadata and refresh/update the catalog index
    addCatalogKeys(self,out)

    return out.getvalue()
