"""A document type for video  submission call outs"""
#  Copyright (c) 2007 EngageMedia
#	andy nicholson andy@engagemedia.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

__author__  = 'andy@engagemedia.org'
__docformat__ = 'plaintext'

# Core
from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import View
from Products.CMFCore.WorkflowCore import WorkflowException
import zLOG

#Arch
try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *

from Products.Archetypes.Marshall import PrimaryFieldMarshaller

# ATCT
from Products.ATContentTypes.content.newsitem import ATNewsItem
from Products.ATContentTypes.content.newsitem import ATNewsItemSchema
from Products.ATContentTypes.content.newsitem import finalizeATCTSchema
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget

#ATVocabularyManager
from Products.ATVocabularyManager.namedvocabulary import NamedVocabulary
#ExternalStorage
from Products.ExternalStorage.ExternalStorage import ExternalStorage

# PressRoom
from Products.PressRoom.content.PressRelease import PressRelease

#our config
from Products.ATVideo.config import *

# Schema declaration
schema = ATNewsItemSchema.copy() +  getattr(PressRelease, 'schema', Schema(())).copy()  

schema['title'].widget.label = 'Title'
schema['title'].widget.i18n_domain = 'submission'

schema = schema + Schema((

    DateTimeField('closingDate',
                required=1,
                searchable=0,
                primary=False,
                languageIndependent=0,
                index="DateIndex:brains",
                widget=CalendarWidget(
                    label='Closing date',
                    label_msgid = "label_closing_date",
                    description='Provide a date for when this callout will be closed.',
                    description_msgid = "help_close_date",
                    i18n_domain = "submission",),
                ),
    LinesField(
        name='Categories',
        widget=MultiSelectionWidget(
            label="Categories. Hold down CTRL/COMMAND and click to multiple select topics.",
            label_msgid='Callout_label_Categories',
            i18n_domain='submission',
            size=10,
        ),
        languageIndependent=1,
        multiValued=1,
        vocabulary=NamedVocabulary("""submission_categories"""),
        accessor="getCategories",
    	    ),

    #FileField(
    #    name='document',
    #    widget=FileWidget
    #    (
    #        label="File",
    #        description="Upload a file (eg entry form).",
    #        label_msgid='Callout_label_document',
    #        description_msgid='Callout_help_document',
    #        i18n_domain='submission',
    #    ),
    #    languageIndependent=1,
    #    storage=ExternalStorage(prefix=FS_DIRECTORY, archive=False, rename=True),
    #    primary=0,
    #    required=0
    #),



    ),)

#A string field ,from PressRelease, which is typically used to describe timings of press release: eg immediate release, for release on Xth blah
schema.moveField('releaseTiming', before='title')
#Hidden on edit form
schema['releaseTiming'].widget.visible['edit'] = 'invisible'
schema['releaseTiming'].widget.label = 'Provide a date for when this callout will be open'
schema['releaseTiming'].widget.i18n_domain = 'submission'


schema.moveField('subhead', after='title')
schema['subhead'].widget.description = 'The callout subheading'
schema['subhead'].widget.label = 'Subheading'
schema['subhead'].widget.i18n_domain = 'submission'
schema['subhead'].widget.visible['edit'] = 'invisible'

schema.moveField('location', after='subhead')
schema['location'].widget.label = 'Location'
schema['location'].widget.description = 'Typically a callout has an associated location in a common format (i.e. City, State)'
schema['location'].widget.i18n_domain = 'submission'

#date the submission is open
schema.moveField('releaseDate', after='location')
schema['releaseDate'].widget.show_hm = False
schema['releaseDate'].widget.label = 'Opening Date'
schema['releaseDate'].widget.description = 'Provide a date for when this callout will be opened'
schema['releaseDate'].widget.i18n_domain = 'submission'
#hidden on edit form.
schema['releaseDate'].widget.visible['edit'] = 'invisible'
schema['releaseDate'].required = False

# closing date for submissions
schema.moveField('closingDate', after='releaseDate')
schema['closingDate'].widget.show_hm = False

schema['image'].widget.description = 'Will be shown in the call out listings, and the call out item itself. Image will be scaled to a sensible size' 
schema['image'].widget.i18n_domain = 'submission'

schema.moveField('Categories', before='text')

#schema[''].widget.visible['edit'] = 'invisible'
schema['releaseContacts'].widget.visible['edit'] = 'invisible'

finalizeATCTSchema(schema)

class Callout(PressRelease):
  """For a video submission call out"""

  meta_type = portal_type = 'Call Out'
  archetype_name  = 'Call Out'
  immediate_view  = 'callout_view'
  default_view    = 'callout_view'
  content_icon    = 'submission_icon.gif'
  typeDescription = """A call out"""
  typeDescMsgId   = """submission_description_edit"""
  schema          = schema

  # Make sure we get title-to-id generation when an object is created
  _at_rename_after_creation = True

  # Get the standard actions (tabs)
  actions = ATNewsItem.actions

  # Make sure we get all the interface declarations from ATNewsItem,
  # which includes support for ISelectableBrowserDefault to get the
  # 'display' menu to work, IHistoryAware and other standard interfaces.
  __implements__ = ATNewsItem.__implements__

  # enable FTP/WebDAV and friends
  PUT = ATNewsItem.PUT
  exclude_from_nav = True

  security       = ClassSecurityInfo()

  security.declareProtected(View, 'getStorydate')
  def getStorydate(self):
    """To bring Releases and Clips into synch in terms of date fields"""
    return self.getReleaseDate()

  security.declarePublic('getCategories')
  def getCategories(self):
    """The submission categories """
    vocab_key = schema['Categories'].get(self)
    return vocab_key

  def processForm(self, data=1, metadata=0, REQUEST=None, values=None):
    """Process the schema looking for data in the form"""
    #call up
    PressRelease.processForm(self,data,metadata,REQUEST,values)
    # move automatically into 'pending' state
    try:
      workflow = getToolByName(self, 'portal_workflow')
      state = workflow.getInfoFor(self,'review_state')
      zLOG.LOG('Callout',zLOG.INFO,' Callout in workflow state %s'%state)
      #dont try to resubmit if already published.
      if not state == 'published':
        workflow.doActionFor(self, 'submit')
    except WorkflowException:
      #XXX just pass ?
      pass


registerType(Callout)
