##################################################################################
#
#    Copyright (C) 2006 Utah State University, All rights reserved.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##################################################################################

__author__  = '''Brent Lambert, David Ray, Jon Thomas'''
__docformat__ = 'plaintext'
__version__   = '$ Revision 0.0 $'[11:-2]

try:
    from Products.Five.browser import BrowserView
except ImportError:
    from zope.app.publisher.browser import BrowserView
from Products.CMFPlone.utils import getToolByName
from urlparse import urlsplit
from xml.dom import minidom
from string import split, find
import urllib


class CopyrightBylineView(BrowserView):
    """ Render the copyright byline """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def getLicenseByline(self):
        """ Get the license byline fields for an object. """
        copyright = self.context.Rights()
        cltool = getToolByName(self.context, 'portal_contentlicensing')
        if not copyright:
            copyright = cltool.DefaultSiteCopyright
        holder, license = cltool.getLicenseAndHolderFromObject(self.context)
        if '(site default)' == holder:
            holder = cltool.DefaultSiteCopyrightHolder
        if 'Site Default' == license[0]:
            license = cltool.DefaultSiteLicense
        license_name = license[1]
        if not license_name or 'None' == license_name:
            license_name = ''
        if 'Creative Commons License' == license[0]:
            license_name = license[0]
        license_url = license[2]
        if not license_url or 'None' == license_url:
            license_url = ''
        license_button = license[3]
        if not license_button or 'None' == license_button:
            license_button = ''
        return copyright, holder, license_name, license_url, license_button

    def getCitationInfo(self):
        """ Gets the citation information """

        # Title
        title = self.context.title

        # Creators
        creator = ''
        index = 1
        
        names = [name.strip() for name in self.context.Creators()]
        
        for cr in names:
            cr = cr.decode('utf-8')
            inits = ''
            crs = []

            crs = cr.split(' ')
            
            for part in crs[:-1]:
                inits += ' ' + part[0] + '.'   
                
            creator += crs[-1]
            if inits:
                creator += "," + inits
            creator += ', '
            index += 1
            
        if creator:
            creator = creator[:-2]
            if creator:
                if creator[-1] != '.':
                    creator += '.'
         
        id = self.context.getId()
        
        portal_url = getToolByName(self.context, 'portal_url')
        portal_name = portal_url.getPortalObject().Title().decode('utf-8')
        
        create_date = self.context.creation_date.strftime('%Y, %B %d')
        
          
        url = self.context.absolute_url()

        import datetime
        date = datetime.date.today().strftime('%B %d, %Y')
        
        if creator:
            prompt_text = "%s (%s). %s. Retrieved %s, from %s Web site: %s." %(creator,create_date,title,date,portal_name,url)
        else:
            prompt_text = "%s. (%s). Retrieved %s, from %s Web site: %s." %(title,create_date,date,portal_name,url)

        return prompt_text.replace('\'','\\\'').replace('\"','\\\'')

class RDFMetadataView(BrowserView):
    """ Express Dublin Core As Rdf  """

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.document = minidom.Document()
        self.cltool = getToolByName(context, 'portal_contentlicensing')
        self.holder, self.license = self.cltool.getLicenseAndHolderFromObject(context)
        

    def writeRDF(self):
        """ Write RDF metadata """
        if 'Creative Commons License' == self.license[0] or ('Creative Commons License' == self.cltool.DefaultSiteLicense[0] and 'Site Default' == self.license[0]):
            if self.license[0] == 'Site Default':
                self.license = self.cltool.DefaultSiteLicense
            data = self.getCCLicenseRDF()
        else:
            data = self.getRDFMetadata()
        # Remove the XML header
        index = find(data, '\n')
        if (index > -1):
            data = data[index + 1:]
        return data

    def getRDFMetadata(self):
        """ Write metadata fields as RDF. """
        rdf_node = self._createNode(self.document, 'rdf:RDF',
                       attrs=[('xmlns:rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#'),
                              ('xmlns:dc', 'http://purl.org/dc/elements/1.1/'),
                              ('xmlns:dcterms', 'http://purl.org/dc/terms')])
        desc_node = self._createNode(rdf_node, 'rdf:Description',
                        attrs=[('rdf:about', self.context.renderBase())])
        self._writeDCMetadata(desc_node)
        return self.document.toprettyxml()        
                                

    def getCCLicenseRDF(self):
        """ Write into RDF CC License elements. """

        holder, license = self.cltool.getLicenseAndHolderFromObject(self.context)
        licenseId = ''
        if len(self.license) >= 3:
            lid = self.license[1].split(' ')
            licenseId = lid[0]

        if licenseId and self.cltool.cc_license_rdf.has_key(licenseId):
            cc_rdf = self.cltool.cc_license_rdf[licenseId]
        
            rdf_node = self._createNode(self.document, 'rdf:RDF',
                           attrs=[('xmlns', 'http://web.resource.org/cc/'),
                                  ('xmlns:dc', 'http://purl.org.dc/elements/1.1/'),
                                  ('xmlns:rdf', 'http://www.w3.org/1999/02/22-rdf-syntax-ns#')])
            work_node = self._createNode(rdf_node, 'Work', attrs=[('rdf:about',
                                                                   self.context.renderBase())])
            self._writeDCMetadata(work_node)
            self._createNode(work_node, 'license', attrs=[('rdf:resource', self.license[2])])
            license_node = self._createNode(rdf_node, 'License',
                               attrs=[('rdf:about', self.license[2])])
            for x in cc_rdf['requires']:
                self._createNode(license_node, 'requires', attrs=[('rdf:resource', x)])
            for x in cc_rdf['permits']:
                self._createNode(license_node, 'permits', attrs=[('rdf:resource', x)]) 
            for x in cc_rdf['prohibits']:
                self._createNode(license_node, 'prohibits', attrs=[('rdf:resource', x)]) 
            return self.document.toprettyxml()
        else:
            return ''

    def _writeDCMetadata(self, node):
        """ Write the dublin core metadata in RDF. """

        # Identifier
        self._createNode(node, 'dc:identifier', self.context.renderBase())

        # Title
        self._createNode(node, 'dc:title', self.context.Title())

        # Language
        lang = self.context.Language()
        if not lang:
            po = self.context.portal_url.getPortalObject()
            lang = po.portal_properties.site_properties.getProperty('default_language')
        self._createNode(node, 'dc:language', lang)

        # Description
        desc = self.context.Description()
        if desc:
            self._createNode(node, 'dc:description', self.context.Description())

        # Subject
        self._renderList(node, 'dc:subject', self.context.Subject())

        # Type
        self._createNode(node, 'dc:type', self.context.Type())

        # Creators
        self._renderList(node, 'dc:creator', self.context.Creators())

        # Contributors
        self._renderList(node, 'dc:contributor', self.context.Contributors())

        # Publisher
        self._createNode(node, 'dc:publisher', self.context.portal_url.getPortalObject().Publisher())

        # Format
        self._createNode(node, 'dc:format', self.context.Format())

        # Rights
        rights = self.context.Rights()
        if not rights:
            rights = self.cltool.DefaultSiteCopyright
        self._createNode(node, 'dc:rights', rights)

    def _renderList(self, node, element, value):
        """ Rendoer a list of items in RDF. """
        if value:
            if len(value) > 1:
                value_node = self._createNode(node, element)
                bag_node = self._createNode(value_node, 'rdf:Bag')
                for x in value:
                    self._createNode(bag_node, 'rdf:li', x)
            else:
                self._createNode(node, element, value[0])
        

    def _createNode(self, parent, ename, value=None, attrs=None):
        """ Create a node in the document. """
        newNode = self.document.createElement(ename)
        parent.appendChild(newNode)
        if value:
            newNode.appendChild(self.document.createTextNode(value))
        if attrs:
            for x in attrs:
                newNode.setAttribute(x[0], x[1])
        return newNode

class RSSView:
    """ Implements base view functionality. """

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.cclicenses = {}

    def getRSSObjects(self):
        """ Get objects for RSS. """
        return []

    def getCCLicense(self, obj):
        """ Get License information """
        holder = self.context.portal_contentlicensing.DefaultSiteCopyrightHolder
        license = self.context.portal_contentlicensing.DefaultSiteLicense
        if obj != self.context.portal_url.getPortalObject():
            result = self.context.portal_contentlicensing.getLicenseAndHolderFromObject(obj)
            if result:
                if result[0] != '(site default)':
                    holder = license[0]
                if result[1][0] != 'Site Default':
                    license = result[1]
        if license and 'Creative Commons License' == license[0]:
            if not self.cclicenses.has_key(license[2]):
                self.cclicenses[license[2]] = license[1].split(' ')[0]
            return license[2], holder
        else:
            return None

    def getCCLicenseTags(self):
        """ Get the list of CC Licenses listed in the RSS Feed. """
        return [(self.cclicenses[x],x) for x in self.cclicenses]

    def getCCLicenseTag(self, cclicense, tag):
        """ Get appropriate cc license entries for the license tag. """
        cc = self.context.portal_contentlicensing.cc_license_rdf
        if cc.has_key(cclicense):
            if cc[cclicense].has_key(tag):
                return cc[cclicense][tag]
        return []
    
