################################################################################
## Copyright (c) 2004-2006, Partecs <info@partecs.com>

## This library is free software; you can redistribute it and/or
## modify it under the terms of the GNU  General Public
## License as published by the Free Software Foundation; either
## version 2 of the License, or any later version.

## This library is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  General Public License for more details.

## You should have received a copy of the GNU  General Public
## License along with this library; if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
################################################################################

"""
Plone Captcha Tool
"""
import random
import md5

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Products.CMFCore.utils import UniqueObject, SimpleItemWithProperties

letters = "abcdefghijklmnopqrstuvwxyz"
letters += letters.upper () + "0123456789"

class PloneCaptcha(UniqueObject, SimpleItemWithProperties):
    """
    Plone Captcha Tool
    """

    id = 'portal_captcha'
    meta_type = 'Plone Captcha tool'
    plone_tool = 1
    security = ClassSecurityInfo()

    _properties = (
        { 'id' : 'captcha_username', 'type' : 'string', 'mode' : 'w', 'label' : 'The username for the captchas.net service' },
        { 'id' : 'captcha_password', 'type' : 'string', 'mode' : 'w', 'label' : 'The password for the captchas.net service' },
        { 'id' : 'captcha_alphabet', 'type' : 'string', 'mode' : 'w', 'label' : 'The alphabet to use for the letters in the captcha image' },
        { 'id' : 'captcha_letters', 'type' : 'int', 'mode' : 'w', 'label' : 'The number of letters in the captcha image' },
        { 'id' : 'captcha_image_width', 'type' : 'int', 'mode' : 'w', 'label' : 'The captcha image width in px' },
        { 'id' : 'captcha_image_height', 'type' : 'int', 'mode' : 'w', 'label' : 'The captcha image height in px' },
        { 'id' : 'captcha_enable_audio', 'type' : 'boolean', 'mode' : 'w', 'label' : 'Add link with phonetic spelling of the captcha' },
    )

    captcha_username = 'demo'
    captcha_password = 'secret'
    captcha_alphabet = 'abcdefghijklmnopqrstuvwxyz'
    captcha_letters = 6
    captcha_image_width = 240
    captcha_image_height = 80
    captcha_enable_audio = True

    security.declarePublic('getImageLink')
    def getImageLink(self, random_string, base_url='http://image.captchas.net'):
        """ Get the link of the captcha image """
        
        return '%s?client=%s&random=%s&alphabet=%s&letters=%s&width=%s&height=%s' % (base_url, self.captcha_username, random_string, self.captcha_alphabet, self.captcha_letters, self.captcha_image_width, self.captcha_image_height)
    
    security.declarePublic('getAudioLink')
    def getAudioLink(self, random_string, base_url='http://audio.captchas.net'):
        """ Get the link of the phonetic spelling of the captcha """
        
        return self.captcha_enable_audio and '%s?client=%s&random=%s' % (base_url, self.captcha_username, random_string) or ''

    security.declarePublic('getCaptchaCode')
    def getCaptchaCode(self):
        '''Returns a random captcha code'''

        random_string = ''
        for i in range(50):  
            random_string += random.choice(letters)
        return random_string 
    
    security.declarePublic('getGeneratedCaptchaCode')        
    def getGeneratedCaptchaCode(self, random_string):
        """ Return Generated Code """

        # The following code seems to work more correctly with modified alphabet and number of letters
        # Parts of this code where taken from http://captchas.net/sample/python/captchasdotnet

        code = self.captcha_password + random_string
        if self.captcha_alphabet != 'abcdefghijklmnopqrstuvwxyz' or self.captcha_letters != 6:
            code += ':' + self.captcha_alphabet + ':' + str(self.captcha_letters)
        
        code_md5 = md5.new(code).digest()
        generated_code = ''
        for p in range(self.captcha_letters):
            n = ord(code_md5[p]) % len(self.captcha_alphabet)
            generated_code += self.captcha_alphabet[n]

        return generated_code

InitializeClass(PloneCaptcha)
