# -*- coding: utf-8 -*-
from zope.interface import Interface, Attribute
from zope.app.publisher.interfaces.browser import IBrowserView

class ITopicsProvider( Interface ):
    u"""The browser views implementing this interface have for mission
    to retrieve a list of categories with all the linked infos from the
    adapted object.
    """
    def get_categories_info(categories):
        u"""Returns a list of dict representing a category.
        The dict contains 3 strings : title, url and id.
        @params : iterable of category ids
        """

    __parent__ = Attribute(
        """The view the provider appears in.

        The view is the third discriminator of the content provider. It allows
        that the content can be controlled for different views.

        Having it stored as the parent is also very important for the security
        context to be kept.
        """)

class IFeaturedVideosRetriever( IBrowserView, ITopicsProvider ):
    u"""This interface defines a featured videos retriever.
    It will fetch the lastest reviewed video, the news and
    the list of the latest videos on the site.
    """
    portal_url = Attribute(u"The url of the portal root.")
    
    featured_items = Attribute(u"The current highlighted items.")
    
    news_and_events = Attribute(u"A list of events and news to display.")
    
    latest_videos = Attribute(u"A list of the latest videos to display.")
    
    featured_video_url = Attribute(u"The url of the page for the featured "
                                   u"videos.")
    
    listing_video_url = Attribute(u"The url of the page listing all the "
                                  u"videos.")
        
class IAuthorPage( Interface ):
    u"""This interface defines a page that is meant to grab an author's
    items and infos out of the catalog and membership tool.
    """
    author = Attribute(u"The id of the author.")
    videos = Attribute(u"A list of the author's videos to display.")
    portrait = Attribute(u"The portrait of the author")
    member_info = Attribute(u"The full infos about the author.")
    homefolder = Attribute(u"The homefolder, if it exists, of the author.")


class IPlumiVideoBrain( Interface ):
    u"""Video Brain Renderer. This multiadapter takes care of rendering a brain
    representing a video, in its context. The context has to be category-aware.
    There are two ways of rendering the brain. See below.
    """
    video = Attribute(u"The video being rendered.")
    video_title = Attribute(u"the title of the video.")
    url = Attribute(u"the url of the video.")
    categories = Attribute(u"A list of the video categories. "
                           u"See ITopicsProvider for more details")

    def render():
        u"""Renders the video brain using the template. Calling this
        method will render the template passing the option 'show_title'
        as False, so the title of the video won't be rendered.
        """

    def render_listing():
        u"""Renders the video brain using the template. Calling this
        method will render the template passing the option 'show_title'
        as True, so the title of the video will be rendered. It is mainly
        used in listings, as the name suggest.
        """
