# -*- coding: utf-8 -*-

# Five & zope3 thingies
from zope import i18n
from zope.interface import implements
from Products.Five  import BrowserView

# CMF
from Products.CMFCore.utils import getToolByName

# Locally
from interfaces import ISteamer

# Internationalization
_ = i18n.MessageIDFactory("vaporisation")


class Steamer( BrowserView ):
    """
    This is the controler for all the actions regarding the cloud.
    It provides the public and specific methods to manipulate the vaporisation.
    Read the interface for more details.
    """
    implements( ISteamer )
    
    def __init__(self, context, request):
        self.context  = context
        self.request  = request
        self.subjects = None
        self.portal_url = getToolByName(context, 'portal_url')()

        putils = getToolByName(context, 'plone_utils')
        self.encoding = putils.getSiteEncoding()
        

    def update_tags_tree(self):
        """This is the method to trigger the cloud update"""
        
        url      = self.request['HTTP_REFERER']
        utils    = getToolByName(self.context, 'plone_utils')
        response = self.request.RESPONSE

        self.context._setTree()
        msg = _(u"The entire cloud has been updated.")
        utils.addPortalMessage(msg)
        return response.redirect(url)


    def getVaporizedCloud(self):
        return self.context._getVaporizedCloudFor(self.subjects is not None
                                                  and self.subjects
                                                  or self.currentSubjects())


    def isJointNavigation(self):
        return self.context.joint


    def currentSubjects(self):
        subjects = self.request.form.get('Subject', None)
        if subjects:

            if isinstance(subjects, str):
                subjects = (subjects,)
            
            encoding = self.encoding
            self.subjects = [unicode(k, encoding)
                             for k in subjects]
        else:
            self.subjects = list()
        return self.subjects


    def removableTags(self):        
        tags = (self.subjects is not None and self.subjects
                or self.currentSubjects())
        
        if not tags:
            return None

        if len(tags) == 1:
            return (dict(name=tags.pop(),
                         link="%s" % self.context.absolute_url()),)
        
        removable = list()
        base_url  = 'search?Subject_usage:ignore_empty=operator:and'
        query     = '%s/%s' % (self.portal_url, base_url)
        tag_url   = '&Subject:list=%s'
        
        for tag in tags:
            tags_url = '&'.join([tag_url % k for k in tags if k != tag])
            removable.append(
                dict(name=tag,
                     link="%s%s" % (query, tags_url))
                )
        return removable


    def getLinkPath(self):
        
        if not self.context.joint:
             return ("%s/search?Subject_usage:ignore_empty=operator:"
                     "and&Subject:list="
                     % self.portal_url)

        tags  = self.subjects
        query = self.request['QUERY_STRING']
        
        if query:
            return ("%s/search?%s&Subject:list=" %
                    (self.portal_url, query))
        
        return ("%s/search?Subject_usage:ignore_empty=operator:and"
                "&Subject:list=" % self.portal_url)
